#!/usr/bin/env python3

import sys
import re
import datetime
import socket
import urllib.parse

emptypage = "https://www.ida.liu.se/edu/ugrad/datortenta/empty"
errorpage = "https://www.ida.liu.se/edu/ugrad/datortenta/blocked.php"
filterlog = "/var/log/squid/filter.log"
basedir = "/usr/libexec/squid/helpers"
hostname = socket.gethostname()

def block_response(url):
    if re.match('.*\.s?html([#?].*)?', url):
        url = urllib.parse.quote(url)
        return 'OK rewrite-url="{}?url={}"'.format(errorpage, url)
    else:
        return 'OK rewrite-url="{}"'.format(emptypage, url)

    # return "OK status=302 url=\"" + errorpage + "\""
    # "status" can only be used in the "Using an HTTP redirector" case,
    # not in the "Using a re-writer to mangle the URL as it passes" case
    # RTFM: https://wiki.squid-cache.org/Features/Redirectors
    
def modify_url(line, ruleset):
### [channel-ID <SP>] URL <SP> client_ip "/" fqdn <SP> user <SP> method [<SP> kv-pairs]<NL>
    list = line.split(' ')

    if list[0].isdigit():
        url = list[1]
    else:
        url = list[0]

    for i in list[2:]:
        if i == "CONNECT":
            return "OK"
        
    if url == "https://www.ida.liu.se/~examadm/start.html":
        return "OK rewrite-url=\"https://www.ida.liu.se/edu/ugrad/datortenta/start.html"
        
    r = re.match('https://www\.ida\.liu\.se/~opendsa/(.*)', url)
    if r != None:
        return 'OK rewrite-url="http://{}:12000/{}"'.format(hostname, r.group(1))

    for rule in ruleset:
        pattern = 0
        allowed = 1
        if rule[pattern].match(url) != None:

            if rule[allowed]:
                return "OK"
            else:
                return block_response(url)

    return block_response(url)
    


def load_rules(ruleset, filename):
    with open(filename, 'r') as f:

        for line in f:
            if not re.match('(^#.*)|(^//.*)|(^;.*)|(^--.*)|^ *$', line):
                columns = line.strip().split()
                ruleset.append( [ re.compile(columns[0]), columns[1].lower() == 'true' ] )
#              ruleset.append( [ columns[0], columns[1].lower() == 'true' ] )


def main():
    
    ruleset = list()
    load_rules(ruleset, basedir + "/rules.d/default.rules")
    load_rules(ruleset, basedir + "/rules.d/rstudio.rules")
    load_rules(ruleset, basedir + "/rules.d/cplusplus.rules")
    load_rules(ruleset, basedir + "/rules.d/opendsa.rules")
    load_rules(ruleset, basedir + "/rules.d/python.rules")
    load_rules(ruleset, basedir + "/rules.d/java.rules")
    load_rules(ruleset, basedir + "/rules.d/ruby.rules")
    load_rules(ruleset, basedir + "/rules.d/sas.rules")
#    load_rules(ruleset, basedir + "/rules.d/dn.rules")

    with open(filterlog, 'a') as log:
        try:
            while True:
            
                line = sys.stdin.readline().strip()

                log.write('{}: {}\n'.format(datetime.datetime.now().strftime("%Y-%m-%d_%H-%M"), line))
                log.flush()

                log.write("From: " + line + "\n")
                log.flush()
            
                new_url = modify_url(line, ruleset)

                log.write("To: " + new_url + "\n")
                log.flush()

                sys.stdout.write(new_url + '\n')
                sys.stdout.flush()
        except Exception as e:
            sys.stderr.write( e )
            sys.stderr.flush()

main()
